<?php
/**
 * InvalidCache deals with storing details about recordings that started with
 * invalid event monitors.
 *
 * Recordings that start with invalid event monitors are tracked and ignored by
 * MRE.  If a recording starts that is to be monitored and never has a valid
 * event monitor attached to it in the first 3 sleep intervals then the
 * recording is simply ignored by MRE.  This prevent the situation where a
 * recording can never have a valid event monitor and the user has setup MRE to
 * ignore invalid event monitors and therefore the event will be recorded for an
 * extra eight hours.  The motivation behind this is ESPN Sunday Night Baseball.
 * Sometimes the subtitle for this game is 'Teams TBA'.  If I decide to record
 * the game then the event monitor will always be invalid for this recording.
 * However, I've setup MRE to ignore invalid event monitors and consider the
 * event still in progress.  Why?  Sometimes an event monitor will fail due to
 * load on the web site providing the data (ESPN).  In these rare cases I
 * simply want to ignore that and extend the recording, if needed.  However, in
 * this case the event monitor will always fail and so MRE should just ignore
 * it.  This is why I've introduced this InvalidCache.  Now I check recordings
 * when they first start.  I now check the event monitor for a recording during
 * the first 3 sleep intervals of the recording.  If during that time the event
 * monitor is invalid then the event is cached by MRE and ignored meaning that
 * MRE will not act on the recording, regardless of preferences.  This will
 * safeguard against 12 hour recordings in cases such as the Sunday Night
 * Baseball example just described.
 *
 * @version $Id: InvalidCache.php,v 1.1 2006/08/13 21:48:57 dbattams Exp $
 * @author Derek Battams <derek@battams.ca>
 * @package MRE
 * @copyright 2006 GPL v2
 */

/**
 * Implement the ignore cache for MRE
 *
 * @package MRE
 */
class InvalidCache
{
   /**
    * Return the cache to the caller in a friendly to use format
    * @return array The ignore cache as a user friendly data structure
    */
   static public function getCache()
   {
      $cache = self::readCache();
      $data = array();
      foreach(array_keys($cache) as $key)
         $data[] = explode('_', $key);
      return $data;
   }

   /**
    * Given a Recording object, add it to the MRE ignore cache
    * @param Recording A valid Recording object
    * @return void
    */
   static public function add(Recording $r)
   {
      $cache = self::readCache();
      $key = $r->getVal('chanid') . '_' . $r->getVal('starttime');
      if(!array_key_exists($key, $cache))
      {
         $cache[$key] = time();
         echo $r->getVal('title') . '[' . $r->getVal('subtitle') . ']';
         echo ": Added to ignore cache due to invalid event monitor.\n";
      }
      self::writeCache($cache);
      return;
   }

   /**
    * Given a Recording object, remove it from the MRE invalid cache
    * @param Recording A valid Recording object
    * @return void
    */
   static public function remove(Recording $r)
   {
      $cache = self::readCache();
      $key = $r->getVal('chanid') . '_' . $r->getVal('starttime');
      if(array_key_exists($key, $cache))
      {
         unset($cache[$key]);
         echo $r->getVal('title') . '[' . $r->getVal('subtitle') . ']';
         echo ": Removed from ignore cache; valid event monitor found.\n";
      }
      self::writeCache($cache);
      return;
   }

   /**
    * Given an MRE cache array, write it disk
    * @param array The MRE invalid cache array
    * @return void
    */
   static protected function writeCache($cache)
   {
      $file = SysConf::getInstance()->getOpt('cache');
      file_put_contents($file, serialize($cache), LOCK_EX);
   }

   /**
    * Read the MRE invalid cache from disk and clean up old entries
    * @return array The up to date MRE invalid cache structure
    */
   static protected function readCache()
   {
      $file = SysConf::getInstance()->getOpt('cache');
      if(file_exists($file))
      {
         $now = time();
         $data = unserialize(file_get_contents($file));
         foreach($data as $key => $ts)
            if(($now - $ts) > 86400)
               unset($data[$key]);
         return $data;
      }
      return array();
   } 
}

?>
